from pyhap.const import STANDALONE_AID
from globals import id_hardware_map
from homekit_accessories.json import HomekitDecoder
import json


def configure_hub_from_local(sensor_hub):
    config_file = open(sensor_hub.config_file_path, "r")
    sensor_hub.config = json.load(config_file)
    config_file.close()

    sensor_hub.serial_no = sensor_hub.config["serial_no"]
    sensor_hub.aid = sensor_hub.config["aid"] or STANDALONE_AID
    sensor_hub.display_name = sensor_hub.config["display_name"] or "OpenHub"


def configure_accessories_from_local(sensor_hub):
    components = sensor_hub.config["accessories"]
    for component in components:
        comp = json.loads(component, cls=HomekitDecoder)
        id_hardware_map.append(comp)



from abc import ABC, abstractmethod
import uuid
from globals import id_hardware_map


class HardwareInterface(ABC):

    def __init__(self, serial_no=uuid.uuid4(), *args, **kwargs):
        self.serial_no = serial_no

        id_hardware_map[str(self.serial_no)] = self

    @abstractmethod
    def create_channel(self):
        pass

    def get_channel(self, channel):
        return self.channels[channel]

    def set_channels(self, channels):
        self.channels = channels




import json

from hardware_interfaces.channels.pi_pico_analog import PiPicoAnalog
from hardware_interfaces.dht22 import DHT22
from hardware_interfaces.mcp3008 import MCP3008
from hardware_interfaces.mod_probe import ModProbe
from hardware_interfaces.pi_pico import PiPico
from hardware_interfaces.veml_7700 import VEML7700
from hardware_interfaces.channels.json.channel_decoder import ChannelDecoder


class HardwareDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
        self.channel_decoder = ChannelDecoder()

    def object_hook(self, dct):
        type = dct['type']

        if type == PiPicoAnalog.__name__:
            json.loads(json.dumps(dct), cls=ChannelDecoder)
        else:

            hardware = None
            if type == 'DHT22':
                hardware = DHT22()
            if type == 'MCP3008':
                hardware = MCP3008()
            elif type == 'ModProbe':
                # hardware = ModProbe(dct['base_dir'],dct['base_dir'],dct['id'])
                hardware = ModProbe(serial_no=dct['id'])
            elif type == 'PiPico':
                hardware = PiPico(dct,None)
            elif type == 'VEML7700':
                hardware = VEML7700()
        return hardware
        # hardware.set_channels(channels)



import busio
import digitalio
import board
import adafruit_mcp3xxx.mcp3008 as MCP
import logging
from hardware_interfaces import hardware_interface
from hardware_interfaces.channels.mcp3008analog import MCP3008Analog


class MCP3008(hardware_interface.HardwareInterface):
    logger = logging.getLogger(__name__)
    channels = []

    def __init__(self, num_channels=3, sck=board.SCK, miso=board.MISO, mosi=board.MOSI, cs_pin=board.D5, serial_no=None,
                 channels=None, *args, **kwargs):
        self.spi = busio.SPI(clock=sck, MISO=miso, MOSI=mosi)
        self.cs = digitalio.DigitalInOut(cs_pin)
        self.mcp = MCP.MCP3008(self.spi, self.cs)
        self.num_channels = num_channels
        self.type = self.__name__

        super().__init__(serial_no, channels, *args, **kwargs)

    def create_channel(self):
        pin = str(input("Which channel is this (0-8)?"))
        return self.create_analog_channel(pin)

    def create_analog_channel(self, channel_index):
        self.logger.debug("Creating Channel: " + str(channel_index))
        channel = MCP3008Analog(mcp=self.mcp, channel_index=channel_index, hardware_serial_no=self.serial_no)
        self.channels.append(channel)
        return channel



import json
from hardware_interfaces.channels import ChannelInterface


class ChannelEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, ChannelInterface):
            dict = o.__dict__
            dict['type'] = o.__name__

        return json.JSONEncoder.default(self, o)



from pyhap.accessory_driver import AccessoryDriver
from pyhap.loader import Loader
# import garden_bridge


accessories = {}
id_hardware_map = {}
picos = []
hardware_id_channels_map = {}
id_channels_map = {}
calibration = {}
hub = None
loader = None
driver = None
#
# def get_bridge(driver):
#     bridge = garden_bridge.GardenBridge(driver)
#     return bridge
#
#

# driver.add_accessory(accessory=get_bridge(driver))



import json
from hardware_interfaces.json import HardwareEncoder
from homekit_accessories.json import HomekitEncoder
from calibrators.json import RawValueEncoder
from OpenHub import HARDWARE_CONFIG_FILE, HOMEKIT_CONFIG_FILE, CALIBRATION_CONFIG_FILE


def write_hardware_config():
    global hardware
    json.dump(hardware, fp=HARDWARE_CONFIG_FILE, cls=HardwareEncoder)


def write_homekit_accessory_config():
    global accessories
    json.dump(open(HOMEKIT_CONFIG_FILE), fp=HOMEKIT_CONFIG_FILE, cls=HomekitEncoder)


def load_calibration_config():
    global calibration
    json.load(open(CALIBRATION_CONFIG_FILE), fp=CALIBRATION_CONFIG_FILE, cls=RawValueEncoder)


def write_configs():
    write_hardware_config()
    write_homekit_accessory_config()
    load_calibration_config()





from globals import id_hardware_map
from hardware_interfaces.json import HardwareDecoder
import json


def configure_hardware_from_local(sensor_hub):
    components = sensor_hub.config["hardware"]
    for component in components:
        comp = json.loads(component, cls=HardwareDecoder)
        id_hardware_map.append(comp)



import logging
from homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface


class AirTemperatureSensor(HomeKitSensorInterface):
    logger = logging.getLogger(__name__)
    run_debug_message = "Current Air Temperature: "

    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, *args, **kwargs):
        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no, *args, **kwargs)

    def set_display_name(self, display_name):
        if self.display_name is None:
            return display_name + "Air Temperature"

    def add_functional_service(self):
        return self.add_preload_service('TemperatureSensor')

    def add_functional_service_characteristic(self):
        return self.service.get_characteristic('CurrentTemperature')
    #
    # async def run(self):
    #
    #     temperature_f = dhti.get_temp_c()
    #     humidity = dhti.get_humidity()
    #     if temperature_f is not None:
    #         self.char_temp.set_value(temperature_f)
    #     if humidity is not None:
    #         self.char_hum.set_value(humidity)
    #



import json
from homekit_accessories.air_temperature import AirTemperatureSensor
from homekit_accessories.hub import Hub
from homekit_accessories.humidity import HumiditySensor
from homekit_accessories.light_sensor import LightSensor
from homekit_accessories.soil_moisture_sensor import SoilMoistureSensor
from homekit_accessories.soil_temperature_sensor import SoilTemperatureSensor
from homekit_accessories.pressure_sensor import PressureSensor
from homekit_accessories.liquid_level_sensor import LiquidLevelSensor
from homekit_accessories.etape import ETapeSensor
from homekit_accessories.pump import Pump

class HomekitDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        type = dct['type']

        if type == AirTemperatureSensor.__name__:
            return AirTemperatureSensor()
        if type == HumiditySensor.__name__:
            return HumiditySensor()
        elif type == LightSensor.__name__:
            return LightSensor()

        elif type == SoilMoistureSensor.__name__:
            return SoilMoistureSensor(dct['id'],dct['display_name'],dct['channels'][0])

        elif type == SoilTemperatureSensor.__name__:
            return SoilTemperatureSensor()
        elif type == PressureSensor.__name__:
            return PressureSensor(dct['id'],dct['display_name'],dct['channels'][0])
        elif type == Pump.__name__:
            return Pump(dct['id'],dct['display_name'],dct['channels'][0])
        elif type == LiquidLevelSensor.__name__:
            return LiquidLevelSensor(dct['id'],dct['display_name'],dct['channels'][0])
        elif type == ETapeSensor.__name__:
            return ETapeSensor(dct['id'],dct['display_name'],dct['channels'][0])
        else:
            return Hub(dct['id'],dct['display_name'])



from hardware_interfaces.channels.channel_interface import ChannelInterface
import logging
from globals import id_hardware_map


class PiPicoPump(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.pipico = id_hardware_map[config['hardware']]
        self.channel_index = config['channel_index']
        self.serial_no = config['id']
        self.type = config['type']
        super().__init__(hardware_serial_no=self.pipico.serial_no, serial_no=self.serial_no, *args, **kwargs)


    async def get_raw_data(self):
        # command = "stat" + str(self.channel_index) + "\n"
        command = "init" + "\n"
        return await self.pipico.send_command(command)

    async def turn_on(self):
        command = "stat" + str(self.channel_index) + "on\n"
        return await self.pipico.send_command(command)

    async def turn_off(self):
        command = "stat" + str(self.channel_index) + "off\n"
        return await self.pipico.send_command(command)



import logging
import garden_bridge
from pyhap.accessory_driver import AccessoryDriver
import signal

logging.basicConfig(level=logging.INFO, format="[%(module)s] %(message)s")
logger = logging.getLogger(__name__)


def calibrate():
    driver = AccessoryDriver(port=51826,
                             persist_file='/home/lesserdaemon/.hap-python/OpenHub/accessory.state')
    signal.signal(signal.SIGTERM, driver.signal_handler)
    bridge = garden_bridge.GardenBridge(driver, 'Bridge')
    logger.info("Setting up new config")
    bridge.configure_first_time()
    channel = input("Which channel would you like to calibrate?")
    bridge.calibrate_sensor(channel)
    bridge.update_config_sensors()


calibrate()
